/**
 * @file tkl_wired.c
 * @brief this file was auto-generated by tuyaos v&v tools, developer can add implements between BEGIN and END
 *
 * @warning: changes between user 'BEGIN' and 'END' will be keeped when run tuyaos v&v tools
 *           changes in other place will be overwrited and lost
 *
 * @copyright Copyright 2020-2021 Tuya Inc. All Rights Reserved.
 *
 */

// --- BEGIN: user defines and implements ---
#include "tkl_wired.h"
#include "tuya_error_code.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <pthread.h>
#include <errno.h>
#include <android/log.h>

#define ROUTE_PATH "/proc/net/route"

#define LOG_TAG "TuyaLog.TKL_WIRED"
#define LOGI(...) __android_log_print(ANDROID_LOG_INFO, LOG_TAG, __VA_ARGS__)
#define LOGD(...) __android_log_print(ANDROID_LOG_DEBUG, LOG_TAG, __VA_ARGS__)
#define LOGE(...) __android_log_print(ANDROID_LOG_ERROR, LOG_TAG, __VA_ARGS__)

static char *net_utils_space_trim( char *str )
{
    int count = 0;
    char *tmp = str;

    if ( !str ) {
        return NULL;
    }

    int size = strlen( str );

    while ( *str == ' ' || (*str >= '\t' && *str <= '\r') ) {
        str++;
        count++;
    }

    if ( count ) {
        memmove( tmp, str, strlen(str) + 1);
    }

    return tmp;
}

// 修改设备名筛选逻辑（兼容 Android 接口命名）
static int net_utils_get_devname(char *devname, int size) {
    FILE *fp = NULL;
    char buf[512];
    char ifname[IFNAMSIZ];
    int ret = -1;

    if (!devname || size <= 0) {
        LOGE("%s %d: arg err\n", __func__, __LINE__);
        return -1;
    }

    fp = fopen("/proc/net/dev", "r");
    if (!fp) {
        LOGE("%s %d: failed to open /proc/net/dev\n", __func__, __LINE__);
        return -1;
    }

    fgets(buf, sizeof(buf), fp);
    fgets(buf, sizeof(buf), fp);

    while (fgets(buf, sizeof(buf), fp)) {
        sscanf(buf, "%[^:]:", ifname);
        // LOGD("ifname: %s\n", ifname);
        char *trimmed = net_utils_space_trim(ifname);
        // LOGD("trimmed: %s\n", trimmed);
        if (!trimmed) continue;

        // 兼容 Android 接口命名
        if (strncmp(trimmed, "en", 2) == 0 ||
            strncmp(trimmed, "eth", 3) == 0 ||
            strncmp(trimmed, "usb", 3) == 0 ||
            strncmp(trimmed, "rndis", 5) == 0||
            strncmp(trimmed, "wlan", 3) == 0) {
            strncpy(devname, trimmed, size);
            ret = 0;
            break;
        }
    }

    fclose(fp);
    return ret;
}

static int get_dns_address(char *dns, size_t size) {
    FILE *fp = NULL;
    char dns1[INET_ADDRSTRLEN] = {0};
    char dns2[INET_ADDRSTRLEN] = {0};

    // 获取 DNS1 地址
    fp = popen("getprop net.dns1", "r");
    if (fp != NULL) {
        fgets(dns1, sizeof(dns1), fp);
        pclose(fp);
        dns1[strcspn(dns1, "\n")] = 0;
    }

    // 获取 DNS2 地址
    fp = popen("getprop net.dns2", "r");
    if (fp != NULL) {
        fgets(dns2, sizeof(dns2), fp);
        pclose(fp);
        dns2[strcspn(dns2, "\n")] = 0; 
    }

    // 设置 DNS 地址
    if (strlen(dns1) > 0) {
        strncpy(dns, dns1, size);
    } else if (strlen(dns2) > 0) {
        strncpy(dns, dns2, size);
    }
    // else {
    //     strncpy(dns, "223.5.5.5", size); // 默认 DNS
    // }

    return 0;
}

static int net_utils_check_cable( const char *ifname, int *plugged )
{
    int sk = -1;
    struct ifreq ifr; 
    int ret = -1;

    if ( !ifname ) {
        LOGE( "%s %d: ifname is null\n", __func__, __LINE__ );
        return -1;
    }

    sk = socket(AF_INET, SOCK_DGRAM, 0);
    if ( sk < 0 ) {
        LOGE( "%s %d: failed to create sk\n", __func__, __LINE__ );
        return -1;
    }

    memset( &ifr, 0, sizeof( ifr ) );
    strncpy( ifr.ifr_name, ifname, sizeof(ifr.ifr_name) - 1 ); 

    ret = ioctl( sk, SIOCGIFFLAGS, &ifr );
    if ( ret < 0 ) {
        LOGE( "%s %d: failed to get iterface info \n", __func__, __LINE__ );
        goto exit_end;
    }

    if ( ifr.ifr_flags & IFF_RUNNING ) {
        *plugged = 1;  // 网线已连接
    } else {
        *plugged = 0;  // 网线未连接
    }

exit_end:
    if ( sk ) {
        close( sk );
    }
    return ret;
}


TKL_WIRED_STATUS_CHANGE_CB status_cb;
/**
 * @brief  set the status change callback
 *
 * @param[in]   cb: the callback when link status changed
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
static void *link_status_thread(void *arg)
{
    TKL_WIRED_STAT_E status;
    int old_status = -1;

    /* Circulate detection of network cable plugging and
       unplugging status, and report status when changing
    */
    while(1) {
        tkl_wired_get_status(&status);
        // LOGD("status: %d\n", status);
        if (status == old_status) {
            sleep(1);
            continue;
        }
        old_status = status;

        status_cb(status);
        sleep(1);
    }
}


static OPERATE_RET tkl_wired_get_gateway(NW_IP_S *ip)
{
    FILE *fp = NULL;
    char *str = NULL;
    char buf[256] = {0};
    unsigned int dest = 0;
    struct in_addr addr;
    int get_gw = 0;

    /* Open file */
    if ((fp = fopen(ROUTE_PATH, "r")) == NULL) {
        LOGE("open %s failed, %d, %s\n", ROUTE_PATH, errno, strerror( errno ));
        return OPRT_COM_ERROR;
    }

    fgets(buf, sizeof(buf), fp);

    /* Read one line at a time, perform string processing */
    while(fgets(buf, sizeof(buf), fp)) {

        sscanf(buf, "%*s%x %x", &dest, &addr.s_addr);
        if ((dest == 0) && (addr.s_addr != 0)) {
            strncpy(ip->gw, inet_ntoa(addr), sizeof(ip->gw));
            get_gw = 1;
            break;
        }
    }

    if (get_gw == 0)
        LOGE("not find gw addr\n");

    fclose(fp);
    return OPRT_OK;
}
// --- END: user defines and implements ---

/**
 * @brief  init create wired link
 *
 * @param[in]   cfg: the configure for wired link
 * 
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
OPERATE_RET tkl_wired_init(TKL_WIRED_BASE_CFG_T *cfg)
{
    // --- BEGIN: user implements ---
    return OPRT_NOT_SUPPORTED;
    // --- END: user implements ---
}

/**
 * @brief  get the link status of wired link
 *
 * @param[out]  is_up: the wired link status is up or not
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
OPERATE_RET tkl_wired_get_status(TKL_WIRED_STAT_E *status) {
    // --- BEGIN: user implements (Android 适配修改) ---
    FILE *fp = NULL;
    char devname[IFNAMSIZ] = {0};
    int sock = -1;
    struct ifreq ifr;
    int ret = -1;
    int plugged = 0;

    // 1. 获取有线网络设备名（兼容 Android 接口命名）
    ret = net_utils_get_devname(devname, sizeof(devname));
    LOGD("net_utils_get_devname: %d\n", ret);
    if (ret < 0) {
        LOGE("%s %d: failed to get devname\n", __func__, __LINE__);
        *status = TKL_WIRED_LINK_DOWN;
        return OPRT_OK;
    }

    // 2. 创建 socket
    if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        LOGE("%s %d: socket create failed (errno=%d)\n", __func__, __LINE__, errno);
        *status = TKL_WIRED_LINK_DOWN;
        return OPRT_SOCK_ERR;
    }

    // 3. 检测物理连接状态
    ret = net_utils_check_cable(devname, &plugged);
    LOGD("net_utils_check_cable: %d , plugged: %d\n", ret, plugged);
    if (ret < 0) {
        close(sock);
        *status = TKL_WIRED_LINK_DOWN;
        return OPRT_COM_ERROR;
    }

    if (!plugged) {
        close(sock);
        *status = TKL_WIRED_LINK_DOWN;
        return OPRT_OK;
    }

    // 4. 检测 IP 地址是否配置
    memset(&ifr, 0, sizeof(ifr));
    strncpy(ifr.ifr_name, devname, sizeof(ifr.ifr_name) - 1);
    // LOGD("ifr_name: %s\n", ifr.ifr_name);
    if (ioctl(sock, SIOCGIFADDR, &ifr) < 0) {
        LOGE("%s %d: failed to get IP (errno=%d)\n", __func__, __LINE__, errno);
        close(sock);
        *status = TKL_WIRED_LINK_DOWN; // 无 IP 时视为链路未完全激活
        return OPRT_OK;
    }

    close(sock);
    *status = TKL_WIRED_LINK_UP;
    return OPRT_OK;
    // --- END: user implements ---
}

/**
 * @brief  set the status change callback
 *
 * @param[in]   cb: the callback when link status changed
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
OPERATE_RET tkl_wired_set_status_cb(TKL_WIRED_STATUS_CHANGE_CB cb)
{
    // --- BEGIN: user implements ---
    pthread_t thread;

    status_cb = cb;

    return pthread_create(&thread, NULL, link_status_thread, NULL);
    // --- END: user implements ---
}

/**
 * @brief  set the ip address of the wired link, used for set the static ipaddress 
 * 
 * @param[in]   ip: the ip address
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
OPERATE_RET tkl_wired_set_ip(NW_IP_S *ip)
{
    // --- BEGIN: user implements ---
    return OPRT_NOT_SUPPORTED;
    // --- END: user implements ---
}

/**
 * @brief  get the ip address of the wired link
 * 
 * @param[in]   ip: the ip address
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
OPERATE_RET tkl_wired_get_ip(NW_IP_S *ip)
{
    // --- BEGIN: user implements ---
    int sock = -1;
    struct ifreq ifr;
    struct sockaddr_in *sin;
    char devname[IFNAMSIZ];
    int ret = -1;

    if (ip == NULL) {
        LOGE("invalid param\n");
        return OPRT_INVALID_PARM;
    }

    // 获取网络接口名称
    ret = net_utils_get_devname( devname, sizeof( devname ) );
    if ( ret < 0 ) {
        LOGE( "%s %d: not get devname\n", __func__, __LINE__ );
        return OPRT_COM_ERROR;
    }

    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        LOGE("socket create failed\n");
         return OPRT_SOCK_ERR;
    }

    memset( ip, 0x0, sizeof( NW_IP_S ) );
    memset(&ifr, 0x0, sizeof(ifr));
    strncpy(ifr.ifr_name, devname, sizeof(ifr.ifr_name) - 1);
    sin = (struct sockaddr_in *)&(ifr.ifr_addr);

    // 获取IP地址
    if (ioctl(sock, SIOCGIFADDR, &ifr) < 0) {
        LOGE( "%s %d: ip ioctl %s, error=%d,%s\n", __func__, __LINE__, devname, errno, strerror( errno ) );
        goto com_error;
    }
    strncpy(ip->ip, inet_ntoa(sin->sin_addr), sizeof(ip->ip));

    // 获取子网掩码
    if (ioctl(sock, SIOCGIFNETMASK, &ifr) < 0) {
        LOGE( "%s %d: mask ioctl %s, error=%d,%s\n", __func__, __LINE__, devname, errno, strerror( errno ) );
        goto com_error;
    }
    strncpy(ip->mask, inet_ntoa(sin->sin_addr), sizeof(ip->mask));

    // 获取网关地址
    if (tkl_wired_get_gateway(ip) < 0)
        goto com_error;

    // 获取DNS地址
    if (get_dns_address(ip->dns, sizeof(ip->dns)) < 0) {
        LOGE("failed to get DNS address\n");
        goto com_error;
    }

    LOGD("mask: %s , ip: %s , devname: %s , gw: %s , dns: %s , dhcpen: %d\n", ip->mask, ip->ip, devname, ip->gw, ip->dns, ip->dhcpen);
    close(sock);
    return OPRT_OK;

com_error:
    // 错误处理：关闭套接字并返回错误
    close(sock);
    return OPRT_COM_ERROR;
    // --- END: user implements ---
}

/**
 * @brief  get the ip address of the wired link
 * 
 * @param[in]   ip: the ip address
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
OPERATE_RET tkl_wired_get_ipv6(NW_IP_TYPE type, NW_IP_S *ip)
{
    // --- BEGIN: user implements ---
    return OPRT_NOT_SUPPORTED;
    // --- END: user implements ---
}

/**
 * @brief  get the mac address of the wired link
 * 
 * @param[in]   mac: the mac address
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
OPERATE_RET tkl_wired_get_mac(NW_MAC_S *mac)
{
    // --- BEGIN: user implements ---
    int i;
    int sock = -1;
    struct ifreq ifr;
    struct sockaddr *addr;
    char devname[IFNAMSIZ];
    int ret = -1;

    if (mac == NULL) {
        LOGE("invalid param\n");
        return OPRT_INVALID_PARM;
    }

    ret = net_utils_get_devname( devname, sizeof( devname ) );
    if ( ret < 0 ) {
        LOGE( "%s %d: not get devname\n", __func__, __LINE__ );
        return OPRT_COM_ERROR;
    }

    LOGD( "%s %d: ifname=%s\n", __func__, __LINE__, devname );

    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
         LOGE("socket create fail\n");
         return OPRT_SOCK_ERR;
    }

    memset(&ifr, 0x0, sizeof(ifr));
    strncpy(ifr.ifr_name, devname, sizeof(ifr.ifr_name) - 1);
    addr = (struct sockaddr *)&ifr.ifr_hwaddr;
    addr->sa_family = 1;

    /* get mac addr */
    if (ioctl(sock, SIOCGIFHWADDR, &ifr) < 0) {
        LOGE( "%s %d: %s get mac, error=%d,%s\n", __func__, __LINE__, devname, errno, strerror( errno ) );
        close(sock);
        return OPRT_COM_ERROR;
    }

    memcpy(mac->mac, addr->sa_data, MAC_ADDR_LEN);

    close(sock);
    return OPRT_OK;
    // --- END: user implements ---
}

/**
 * @brief  set the mac address of the wired link
 * 
 * @param[in]   mac: the mac address
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
OPERATE_RET tkl_wired_set_mac(CONST NW_MAC_S *mac)
{
    // --- BEGIN: user implements ---
    int i;
    int sock = -1;
    struct ifreq ifr;
    struct sockaddr *addr;
    char devname[IFNAMSIZ];
    int ret = -1;

    if (mac == NULL) {
        LOGE("invalid param\n");
        return OPRT_INVALID_PARM;
    }
    LOGD("%s %d: %02x:%02x:%02x:%02x:%02x:%02x\n", __func__, __LINE__, mac->mac[0], mac->mac[1], mac->mac[2], mac->mac[3], mac->mac[4], mac->mac[5] );

    ret = net_utils_get_devname( devname, sizeof( devname ) );
    if ( ret < 0 ) {
        LOGE( "%s %d: not get devname\n", __func__, __LINE__ );
        return OPRT_COM_ERROR;
    }

    LOGD( "%s %d: ifname=%s\n", __func__, __LINE__, devname );

    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        LOGE("socket create fail\n");
         return OPRT_SOCK_ERR;
    }

    memset(&ifr, 0x0, sizeof(ifr));
    strncpy(ifr.ifr_name, devname, sizeof(ifr.ifr_name) - 1);
    addr = (struct sockaddr *)&ifr.ifr_hwaddr;
    addr->sa_family = 1;
    memcpy(addr->sa_data, mac->mac, MAC_ADDR_LEN);

    /* set mac addr */
    if (ioctl(sock, SIOCSIFHWADDR, &ifr) < 0) {
        LOGE( "%s %d: %s set mac, error=%d,%s\n", __func__, __LINE__, devname, errno, strerror( errno ) );
        close(sock);
        return OPRT_COM_ERROR;
    }

    close(sock);
    return OPRT_OK;
    // --- END: user implements ---
}

